package nl.ontwik.services;

import com.amazon.webservices.awsecommerceservice._2009_10_01.AWSECommerceService;
import com.amazon.webservices.awsecommerceservice._2009_10_01.AWSECommerceServicePortType;
import com.amazon.webservices.awsecommerceservice._2009_10_01.EditorialReview;
import com.amazon.webservices.awsecommerceservice._2009_10_01.Item;
import com.amazon.webservices.awsecommerceservice._2009_10_01.ItemAttributes;
import com.amazon.webservices.awsecommerceservice._2009_10_01.ItemLookupRequest;


import com.amazon.webservices.awsecommerceservice._2009_10_01.Items;
import java.util.ArrayList;
import java.util.List;

import javax.xml.ws.Holder;
import javax.xml.ws.handler.Handler;
import javax.xml.ws.handler.HandlerResolver;
import javax.xml.ws.handler.PortInfo;
import nl.ontwik.model.Book;
import nl.ontwik.services.BookInformationService;

/**
 * Hello world!
 *
 */
public class AmazonBookAdapter implements BookInformationService {
//System.setProperty("com.sun.xml.ws.transport.http.client.HttpTransportPipe.dump", "true");

    private static final String AWS_ACCESS_KEY_ID = "0BGMA2EY8MR________";

    public Book findByISBN(String isbn) {
        if (isbn == null || isbn.length() != 10) {
            return null;
        }
        AWSECommerceService awsecommerceservice = new AWSECommerceService();
        awsecommerceservice.setHandlerResolver(new HandlerResolver() {

            @Override
            public List<Handler> getHandlerChain(PortInfo portInfo) {
                List<Handler> handlerList = new ArrayList<Handler>();
                handlerList.add(new AmazonSOAPHandler());
                return handlerList;
            }
        });
        AWSECommerceServicePortType awsecommerceserviceport =
                awsecommerceservice.getAWSECommerceServicePort();
        List<ItemLookupRequest> request = createItemLookupRequest(isbn);

        Holder<List<Items>> items = new Holder<List<Items>>();
        awsecommerceserviceport.itemLookup(null, AWS_ACCESS_KEY_ID, null, null,
                null, null, null, request, null, items);

        return parseResults(items);
    }

    public Book parseResults(Holder<List<Items>> holder) {
        Book book = new Book();

        List<Items> itemsList = holder.value;
        List<Item> itemList = itemsList.get(0).getItem();
        if (itemList == null || itemList.size() == 0) {
            return null;
        } else {
            Item item = itemList.get(0);
            ItemAttributes attributes = item.getItemAttributes();

            book.setIsbn(attributes.getISBN());
            book.setEan(attributes.getEAN());
            book.setTitle(attributes.getTitle());
            book.setPublisher(attributes.getPublisher());
            if (attributes.getNumberOfPages() != null) {
                book.setNumberOfPages(attributes.getNumberOfPages().toString());
            }
            book.setEdition(attributes.getEdition());
            book.setPublicationDate(attributes.getPublicationDate());
            if (item.getLargeImage() != null) {
                book.setImageUrl(item.getLargeImage().getURL());
            }
            if (item.getEditorialReviews() != null) {
                List<EditorialReview> reviewList = item.getEditorialReviews().getEditorialReview();
                for (EditorialReview review : reviewList) {
                    if (review.getSource().equals("Product Description")) {
                        book.setDescription(review.getContent());
                    }
                }
            }
            for (String author : attributes.getAuthor()) {
                book.addAuthor(author);
            }
        }
        return book;
    }

    private List<ItemLookupRequest> createItemLookupRequest(String isbn) {
        List<ItemLookupRequest> itemrequests = new ArrayList<ItemLookupRequest>();

        ItemLookupRequest request = new ItemLookupRequest();
        request.setIdType("ASIN");
        request.getItemId().add(isbn);
        request.getResponseGroup().add("EditorialReview");
        request.getResponseGroup().add("Medium");
        itemrequests.add(request);

        return itemrequests;
    }
}
